using System;
using System.Collections.Generic;
using Oxide.Core;
using Oxide.Core.Plugins;
using UnityEngine;

namespace Oxide.Plugins
{
    [Info("DangerousLandmine", "RustFlash", "1.0.2")]
    [Description("Makes landmines more realistic and allows explosions during placement.")]
    public class DangerousLandmine : RustPlugin
    {
        private const string permUse = "dangerouslandmine.use";
        private static System.Random random = new System.Random();

        private class ConfigData
        {
            public float ExplosionChance { get; set; } = 50f; 
            public float ExplosionRadius { get; set; } = 5f; 
            public float ExplosionDamage { get; set; } = 100f; 
        }

        private ConfigData config;

        protected override void LoadDefaultConfig()
        {
            PrintWarning("Creating a new configuration file.");
            config = new ConfigData();
            SaveConfig();
        }

        protected override void LoadConfig()
        {
            base.LoadConfig();
            config = Config.ReadObject<ConfigData>();
        }

        protected override void SaveConfig()
        {
            Config.WriteObject(config);
        }

        private void Init()
        {
            permission.RegisterPermission(permUse, this);
            LoadConfig();
        }

        private void OnEntityBuilt(Planner planner, GameObject gameObject)
        {
            if (gameObject?.ToBaseEntity() is Landmine landmine)
            {
                var player = planner.GetOwnerPlayer();
                if (player != null && permission.UserHasPermission(player.UserIDString, permUse))
                {
                    if (ShouldExplode())
                    {
                        timer.Once(0.1f, () => ExplodeLandmine(landmine));
                    }
                }
            }
        }

        private bool ShouldExplode()
        {
            return random.NextDouble() * 100 < config.ExplosionChance;
        }

        private void ExplodeLandmine(Landmine landmine)
        {
            if (landmine == null) return;

            Effect.server.Run("assets/prefabs/tools/c4/effects/c4_explosion.prefab", landmine.transform.position);

            DealExplosionDamage(landmine.transform.position, config.ExplosionRadius, config.ExplosionDamage);

            landmine.Kill(BaseNetworkable.DestroyMode.Gib);
        }

        private void DealExplosionDamage(Vector3 explosionPosition, float radius, float damage)
        {
            var entities = new List<BaseEntity>();
            Vis.Entities(explosionPosition, radius, entities);

            foreach (var entity in entities)
            {
                var player = entity as BasePlayer;
                if (player != null)
                {
                    var distance = Vector3.Distance(explosionPosition, player.transform.position);
                    var damageMultiplier = 1 - (distance / radius);
                    var finalDamage = damage * damageMultiplier;

                    player.Hurt(finalDamage);
                }
            }
        }
    }
}
